@@ -6,6 +6,7 @@ Provides:
 .. autoclass:: OutputBase
    :members:
    :private-members:
+   :noindex:
 
 .. autoclass:: OutputBasePseudoSpectral
    :members:
@@ -15,13 +16,16 @@ Provides:
    :members:
    :private-members:
 
+.. autoclass:: SimReprMaker
+   :members:
+   :private-members:
+   :noindex:
 """
 
 import datetime
 import os
 import shutil
 import numbers
-from time import sleep
 from copy import copy
 from warnings import warn
 
@@ -34,12 +38,15 @@ from fluiddyn.util import mpi
 from fluiddyn.util import is_run_from_ipython, print_memory_usage
 from fluiddyn.io import FLUIDSIM_PATH, Path
 from fluidsim_core.output import OutputCore, SimReprMakerCore
+from fluidsim_core.params import iter_complete_params
 
 import fluidsim
 from fluidsim.util.util import open_patient
 
 
 class SimReprMaker(SimReprMakerCore):
+    """Produce a string representing the simulation"""
+
     def get_time_as_str(self):
         params = self.sim.params
         if not params.NEW_DIR_RESULTS and (
@@ -132,16 +139,7 @@ are called.
         )
 
         dict_classes = info_solver.classes.Output.import_classes()
-        for Class in list(dict_classes.values()):
-            if hasattr(Class, "_complete_params_with_default"):
-                try:
-                    Class._complete_params_with_default(params)
-                except TypeError:
-                    try:
-                        Class._complete_params_with_default(params, info_solver)
-                    except TypeError as e:
-                        e.args += ("for class: " + repr(Class),)
-                        raise
+        iter_complete_params(params, info_solver, dict_classes.values())
 
     def __init__(self, sim):
         super().__init__(sim)
@@ -255,9 +253,15 @@ are called.
 
     def _save_info_solver_params_xml(self, replace=False):
         """Save files with information on the solver and on the run."""
-        super()._save_info_solver_params_xml(
-            replace=replace, comment=f"FluidSim {fluidsim.get_local_version()}"
-        )
+        if (
+            mpi.rank == 0
+            and self._has_to_save
+            and self.sim.params.NEW_DIR_RESULTS
+        ):
+            super()._save_info_solver_params_xml(
+                replace=replace,
+                comment=f"FluidSim {fluidsim.get_local_version()}",
+            )
 
     def init_with_initialized_state(self):
 
@@ -505,7 +509,7 @@ class SpecificOutput:
         self.t_last_save = self.sim.time_stepping.t
 
     def _online_save(self):
-        """Save the values at one time. """
+        """Save the values at one time."""
         tsim = self.sim.time_stepping.t
         if tsim - self.t_last_save >= self.period_save:
             self.t_last_save = tsim
