@@ -96,11 +96,7 @@ def compute_energy_from_2fields(vx: Ac, vy: Ac):
 
 
 @boost
-def compute_energy_from_3fields(
-    vx: Ac,
-    vy: Ac,
-    vz: Ac,
-):
+def compute_energy_from_3fields(vx: Ac, vy: Ac, vz: Ac):
     return 0.5 * (np.abs(vx) ** 2 + np.abs(vy) ** 2 + np.abs(vz) ** 2)
 
 
@@ -276,11 +272,11 @@ Lx, Ly and Lz: float
             pass
         else:
             if NO_SHEAR_MODES:
-                COND_NOSHEAR = self.Kx ** 2 + self.Ky ** 2 == 0.0
-                where_dealiased = np.logical_or(
-                    COND_NOSHEAR, self.where_dealiased
+                COND_NOSHEAR = self.Kx**2 + self.Ky**2 == 0.0
+                self.where_dealiased = np.array(
+                    np.logical_or(COND_NOSHEAR, self.where_dealiased),
+                    dtype=np.uint8,
                 )
-                self.where_dealiased = np.array(where_dealiased, dtype=np.uint8)
 
     def get_region_multiple_aliases(self):
         aliases_x = abs(self.Kx) >= 2 / 3 * self.deltakx * self.nx / 2
@@ -301,7 +297,7 @@ Lx, Ly and Lz: float
 
     @property
     def K4(self):
-        return self.K2 ** 2
+        return self.K2**2
 
     def build_invariant_arrayX_from_2d_indices12X(self, arr2d, oper2d=None):
         """Build a 3D array from a 2D array"""
@@ -542,7 +538,7 @@ Lx, Ly and Lz: float
         urx_fft, ury_fft contain shear modes!
 
         """
-        inv_Kh_square_nozero = self.Kx ** 2 + self.Ky ** 2
+        inv_Kh_square_nozero = self.Kx**2 + self.Ky**2
         inv_Kh_square_nozero[inv_Kh_square_nozero == 0] = 1e-14
         inv_Kh_square_nozero = 1 / inv_Kh_square_nozero
 
@@ -555,6 +551,273 @@ Lx, Ly and Lz: float
 
         return urx_fft, ury_fft, udx_fft, udy_fft
 
+    @boost
+    def project_kradial3d(self, vx_fft: Ac, vy_fft: Ac, vz_fft: Ac):
+        r"""Project (inplace) a vector field parallel to the k-radial direction of the wavevector.
+
+        Parameters
+        ----------
+
+        Arrays containing the velocity in Fourier space.
+
+        Notes
+        -----
+
+        .. |kk| mathmacro:: \mathbf{k}
+        .. |ee| mathmacro:: \mathbf{e}
+        .. |vv| mathmacro:: \mathbf{v}
+
+        The radial unitary vector for the mode :math:`\kk` is
+
+        .. math::
+
+           \ee_\kk = \frac{\kk}{|\kk|}
+           = \sin \theta_\kk \cos \varphi_\kk ~ \ee_x
+           + \sin \theta_\kk \sin \varphi_\kk ~ \ee_y
+           + \cos \theta_\kk ~ \ee_z,
+
+        and the projection of a velocity mode :math:`\hat{\vv}_\kk` along
+        :math:`\ee_\kk` is
+
+        .. math:: \hat{v}_\kk ~ \ee_\kk \equiv \hat{\vv}_\kk \cdot \ee_\kk ~ \ee_\kk
+
+        This function set :math:`\hat{\vv}_\kk = \hat{v}_\kk ~ \ee_\kk` for all
+        modes.
+
+        .. note:
+
+           For a divergent less vector field, the resulting vector is zero.
+
+        """
+
+        K_square_nozero = self.Kx**2 + self.Ky**2 + self.Kz**2
+        K_square_nozero[K_square_nozero == 0] = 1e-14
+        inv_K_square_nozero = 1.0 / K_square_nozero
+
+        tmp = (
+            self.Kx * vx_fft + self.Ky * vy_fft + self.Kz * vz_fft
+        ) * inv_K_square_nozero
+
+        n0, n1, n2 = vx_fft.shape
+        for i0 in range(n0):
+            for i1 in range(n1):
+                for i2 in range(n2):
+                    vx_fft[i0, i1, i2] = self.Kx[i0, i1, i2] * tmp[i0, i1, i2]
+                    vy_fft[i0, i1, i2] = self.Ky[i0, i1, i2] * tmp[i0, i1, i2]
+                    vz_fft[i0, i1, i2] = self.Kz[i0, i1, i2] * tmp[i0, i1, i2]
+
+    @boost
+    def project_poloidal(self, vx_fft: Ac, vy_fft: Ac, vz_fft: Ac):
+        r"""Project (inplace) a vector field parallel to the k-poloidal (or polar) direction.
+
+        Parameters
+        ----------
+
+        Arrays containing the velocity in Fourier space.
+
+        Notes
+        -----
+
+        The poloidal unitary vector for the mode :math:`\kk` is
+
+        .. math::
+
+           \ee_{\kk\theta}
+           = \cos \theta_\kk \cos \varphi_\kk ~ \ee_x
+           + \cos \theta_\kk \sin \varphi_\kk ~ \ee_y - \sin \theta_\kk ~ \ee_z,
+
+        and the projection of a velocity mode :math:`\hat{\vv}_\kk` along
+        :math:`\ee_{\kk\theta}` is
+
+        .. math::
+
+           \hat{v}_{\kk\theta} ~ \ee_{\kk\theta}
+           \equiv \hat{\vv}_\kk \cdot \ee_{\kk\theta} ~ \ee_{\kk\theta}
+
+        This function set :math:`\hat{\vv}_\kk = \hat{v}_{\kk\theta} ~
+        \ee_{\kk\theta}` for all modes.
+        """
+
+        Kh_square = self.Kx**2 + self.Ky**2
+        K_square_nozero = Kh_square + self.Kz**2
+        Kh_square_nozero = Kh_square.copy()
+
+        Kh_square_nozero[Kh_square_nozero == 0] = 1e-14
+        K_square_nozero[K_square_nozero == 0] = 1e-14
+
+        inv_Kh_square_nozero = 1.0 / Kh_square_nozero
+        inv_K_square_nozero = 1.0 / K_square_nozero
+
+        cos_theta_k = self.Kz * np.sqrt(inv_K_square_nozero)
+        sin_theta_k = np.sqrt(Kh_square * inv_K_square_nozero)
+        cos_phi_k = self.Kx * np.sqrt(inv_Kh_square_nozero)
+        sin_phi_k = self.Ky * np.sqrt(inv_Kh_square_nozero)
+
+        tmp = (
+            cos_theta_k * cos_phi_k * vx_fft
+            + cos_theta_k * sin_phi_k * vy_fft
+            - sin_theta_k * vz_fft
+        )
+
+        n0, n1, n2 = vx_fft.shape
+        for i0 in range(n0):
+            for i1 in range(n1):
+                for i2 in range(n2):
+                    vx_fft[i0, i1, i2] = (
+                        cos_theta_k[i0, i1, i2]
+                        * cos_phi_k[i0, i1, i2]
+                        * tmp[i0, i1, i2]
+                    )
+                    vy_fft[i0, i1, i2] = (
+                        cos_theta_k[i0, i1, i2]
+                        * sin_phi_k[i0, i1, i2]
+                        * tmp[i0, i1, i2]
+                    )
+                    vz_fft[i0, i1, i2] = (
+                        -sin_theta_k[i0, i1, i2] * tmp[i0, i1, i2]
+                    )
+
+    @boost
+    def vpfft_from_vecfft(self, vx_fft: Ac, vy_fft: Ac, vz_fft: Ac):
+        """Return the poloidal component of a vector field."""
+
+        Kh_square = self.Kx**2 + self.Ky**2
+        K_square_nozero = Kh_square + self.Kz**2
+        Kh_square_nozero = Kh_square.copy()
+
+        Kh_square_nozero[Kh_square_nozero == 0] = 1e-14
+        K_square_nozero[K_square_nozero == 0] = 1e-14
+
+        inv_Kh_square_nozero = 1.0 / Kh_square_nozero
+        inv_K_square_nozero = 1.0 / K_square_nozero
+
+        cos_theta_k = self.Kz * np.sqrt(inv_K_square_nozero)
+        sin_theta_k = np.sqrt(Kh_square * inv_K_square_nozero)
+        cos_phi_k = self.Kx * np.sqrt(inv_Kh_square_nozero)
+        sin_phi_k = self.Ky * np.sqrt(inv_Kh_square_nozero)
+
+        result = (
+            cos_theta_k * cos_phi_k * vx_fft
+            + cos_theta_k * sin_phi_k * vy_fft
+            - sin_theta_k * vz_fft
+        )
+
+        return result
+
+    @boost
+    def vecfft_from_vpfft(self, vp_fft: Ac):
+        """Return a vector field from the poloidal component."""
+
+        Kh_square = self.Kx**2 + self.Ky**2
+        K_square_nozero = Kh_square + self.Kz**2
+        Kh_square_nozero = Kh_square.copy()
+
+        Kh_square_nozero[Kh_square_nozero == 0] = 1e-14
+        K_square_nozero[K_square_nozero == 0] = 1e-14
+
+        inv_Kh_square_nozero = 1.0 / Kh_square_nozero
+        inv_K_square_nozero = 1.0 / K_square_nozero
+
+        cos_theta_k = self.Kz * np.sqrt(inv_K_square_nozero)
+        sin_theta_k = np.sqrt(Kh_square * inv_K_square_nozero)
+        cos_phi_k = self.Kx * np.sqrt(inv_Kh_square_nozero)
+        sin_phi_k = self.Ky * np.sqrt(inv_Kh_square_nozero)
+
+        ux_fft = cos_theta_k * cos_phi_k * vp_fft
+        uy_fft = cos_theta_k * sin_phi_k * vp_fft
+        uz_fft = -sin_theta_k * vp_fft
+
+        return ux_fft, uy_fft, uz_fft
+
+    @boost
+    def project_toroidal(self, vx_fft: Ac, vy_fft: Ac, vz_fft: Ac):
+        r"""Project (inplace) a vector field parallel to the k-toroidal (or azimutal) direction.
+
+        Parameters
+        ----------
+
+        Arrays containing the velocity in Fourier space.
+
+        Notes
+        -----
+
+        The toroidal unitary vector for the mode :math:`\kk` is
+
+        .. math::
+
+           \ee_{\kk\varphi}
+           = - \sin \varphi_\kk ~ \ee_x + \cos \varphi_\kk ~ \mathbb{e}_y,
+
+        and the projection of a velocity mode :math:`\hat{\vv}_\kk` along
+        :math:`\ee_{\kk\varphi}` is
+
+        .. math::
+
+           \hat{v}_{\kk\varphi} ~ \ee_{\kk\varphi}
+           \equiv \hat{\vv}_\kk \cdot \ee_{\kk\varphi} ~ \ee_{\kk\varphi}
+
+        This function compute :math:`\hat{\vv}_\kk = \hat{v}_{\kk\varphi} ~
+        \ee_{\kk\varphi}` for all modes.
+        """
+
+        Kh_square_nozero = self.Kx**2 + self.Ky**2
+        Kh_square_nozero[Kh_square_nozero == 0] = 1e-14
+        inv_Kh_square_nozero = 1.0 / Kh_square_nozero
+        del Kh_square_nozero
+
+        tmp = np.sqrt(inv_Kh_square_nozero)
+        cos_phi_k = self.Kx * tmp
+        sin_phi_k = self.Ky * tmp
+
+        tmp = -sin_phi_k * vx_fft + cos_phi_k * vy_fft
+
+        n0, n1, n2 = vx_fft.shape
+        for i0 in range(n0):
+            for i1 in range(n1):
+                for i2 in range(n2):
+                    vx_fft[i0, i1, i2] = -sin_phi_k[i0, i1, i2] * tmp[i0, i1, i2]
+                    vy_fft[i0, i1, i2] = cos_phi_k[i0, i1, i2] * tmp[i0, i1, i2]
+                    vz_fft[i0, i1, i2] = 0.0
+
+    @boost
+    def vtfft_from_vecfft(self, vx_fft: Ac, vy_fft: Ac, vz_fft: Ac):
+        """Return the toroidal component of a vector field."""
+
+        Kh_square_nozero = self.Kx**2 + self.Ky**2
+        Kh_square_nozero[Kh_square_nozero == 0] = 1e-14
+        inv_Kh_square_nozero = 1.0 / Kh_square_nozero
+        del Kh_square_nozero
+
+        tmp = np.sqrt(inv_Kh_square_nozero)
+        cos_phi_k = self.Kx * tmp
+        sin_phi_k = self.Ky * tmp
+
+        result = -sin_phi_k * vx_fft + cos_phi_k * vy_fft
+
+        return result
+
+    @boost
+    def vecfft_from_vtfft(self, vt_fft: Ac):
+        """Return a 3D vector field from the toroidal component."""
+
+        Kh_square = self.Kx**2 + self.Ky**2
+        K_square_nozero = Kh_square + self.Kz**2
+        Kh_square_nozero = Kh_square.copy()
+
+        Kh_square_nozero[Kh_square_nozero == 0] = 1e-14
+        K_square_nozero[K_square_nozero == 0] = 1e-14
+
+        inv_Kh_square_nozero = 1.0 / Kh_square_nozero
+
+        cos_phi_k = self.Kx * np.sqrt(inv_Kh_square_nozero)
+        sin_phi_k = self.Ky * np.sqrt(inv_Kh_square_nozero)
+
+        ux_fft = -sin_phi_k * vt_fft
+        uy_fft = cos_phi_k * vt_fft
+        uz_fft = 0.0 * vt_fft
+
+        return ux_fft, uy_fft, uz_fft
+
     @boost
     def divhfft_from_vxvyfft(self, vx_fft: Ac, vy_fft: Ac):
         """Compute the horizontal divergence in spectral space."""
@@ -563,7 +826,7 @@ Lx, Ly and Lz: float
     @boost
     def vxvyfft_from_rotzfft(self, rotz_fft: Ac):
 
-        inv_Kh_square_nozero = self.Kx ** 2 + self.Ky ** 2
+        inv_Kh_square_nozero = self.Kx**2 + self.Ky**2
         inv_Kh_square_nozero[inv_Kh_square_nozero == 0] = 1e-14
         inv_Kh_square_nozero = 1 / inv_Kh_square_nozero
 
@@ -571,6 +834,17 @@ Lx, Ly and Lz: float
         vy_fft = -1j * self.Kx * inv_Kh_square_nozero * rotz_fft
         return vx_fft, vy_fft
 
+    @boost
+    def vxvyfft_from_divhfft(self, divh_fft: Ac):
+
+        inv_Kh_square_nozero = self.Kx**2 + self.Ky**2
+        inv_Kh_square_nozero[inv_Kh_square_nozero == 0] = 1e-14
+        inv_Kh_square_nozero = 1 / inv_Kh_square_nozero
+
+        vx_fft = -1j * self.Kx * inv_Kh_square_nozero * divh_fft
+        vy_fft = -1j * self.Ky * inv_Kh_square_nozero * divh_fft
+        return vx_fft, vy_fft
+
     def get_grid1d_seq(self, axe="x"):
 
         if axe not in ("x", "y", "z"):
