@@ -80,7 +80,7 @@ class SpectralEnergyBudgetNS2DStrat(SpectralEnergyBudgetBase):
         if self.params.N == 0:
             transferEA_fft = np.zeros_like(transferZ_fft)
         else:
-            transferEA_fft = (1 / self.params.N ** 2) * np.real(
+            transferEA_fft = (1 / self.params.N**2) * np.real(
                 b_fft.conj() * Fb_fft
             )
 
@@ -100,7 +100,7 @@ class SpectralEnergyBudgetNS2DStrat(SpectralEnergyBudgetBase):
         if self.params.N == 0:
             dissEA_fft = np.zeros_like(dissEK_fft)
         else:
-            dissEA_fft = (1 / self.params.N ** 2) * np.real(
+            dissEA_fft = (1 / self.params.N**2) * np.real(
                 freq_diss_EK * (b_fft.conj() * b_fft)
             )
 
@@ -394,7 +394,6 @@ class SpectralEnergyBudgetNS2DStrat(SpectralEnergyBudgetBase):
         ax1.set_xscale("log")
         ax1.set_yscale("linear")
         ax1.set_title("Spectral energy budget\n" + self.output.summary_simul)
-        ax1.set_xlim(left=1e-1, right=5e1)
 
         # Parameters of figure 2
         fig, ax2 = self.output.figure_axe()
@@ -403,7 +402,6 @@ class SpectralEnergyBudgetNS2DStrat(SpectralEnergyBudgetBase):
         ax2.set_xscale("log")
         ax2.set_yscale("linear")
         ax2.set_title("Spectral energy budget\n" + self.output.summary_simul)
-        ax2.set_xlim(left=1e-1, right=5e1)
 
         # Compute time average dissipation to normalize fluxes
         spatial_mean_results = self.output.spatial_means.load()
@@ -417,7 +415,7 @@ class SpectralEnergyBudgetNS2DStrat(SpectralEnergyBudgetBase):
         imax_plot_spatial_times = np.argmin(abs(t - tmax))
 
         eps_tot_time_average = eps_tot[
-            imin_plot_spatial_times:imax_plot_spatial_times
+            imin_plot_spatial_times : imax_plot_spatial_times + 1
         ].mean(0)
 
         # calculate forcing wave-number k_f to normalize k_x and k_z
@@ -437,10 +435,12 @@ class SpectralEnergyBudgetNS2DStrat(SpectralEnergyBudgetBase):
 
             # Band forcing region kx
             k_fxmin = nkmin * self.sim.oper.deltak * np.sin(angle)
+            k_fxmin = max(k_fxmin, self.sim.oper.deltakx)
             k_fxmax = nkmax * self.sim.oper.deltak * np.sin(angle)
 
             # Band forcing region ky
             k_fymin = nkmin * self.sim.oper.deltak * np.cos(angle)
+            k_fymin = max(k_fymin, self.sim.oper.deltaky)
             k_fymax = nkmax * self.sim.oper.deltak * np.cos(angle)
 
             # plot forcing range
@@ -448,7 +448,7 @@ class SpectralEnergyBudgetNS2DStrat(SpectralEnergyBudgetBase):
             ax2.axvspan(k_fymin / k_fy, k_fymax / k_fy, alpha=0.15, color="black")
 
             # Plot ozmidov scale
-            k_o = (self.params.N ** 3 / self.params.forcing.forcing_rate) ** (
+            k_o = (self.params.N**3 / self.params.forcing.forcing_rate) ** (
                 1 / 2
             )
             ax1.axvline(x=k_o / k_fx, color="black", linestyle="--")
