@@ -9,8 +9,9 @@ Provides:
    util
    testing
    console
+   scripts
    mini_oper_modif_resol
 
 """
 
-from .util import times_start_end_from_path
+from .util import times_start_end_from_path, ensure_radians
