@@ -30,6 +30,7 @@ from importlib import import_module
 from pathlib import Path
 from time import perf_counter
 from typing import Union
+from math import radians
 
 import fluiddyn as fld
 import h5netcdf
@@ -68,6 +69,19 @@ import_simul_class_from_key = partial(
 )
 
 
+# A bad hack to silence a h5netcdf warning
+h5netcdf.File = partial(h5netcdf.File, decode_vlen_strings=True)
+# FutureWarning: String decoding changed with h5py >= 3.0. See
+# https://docs.h5py.org/en/latest/strings.html and
+# https://github.com/h5netcdf/h5netcdf/issues/132 for more details. Currently
+# backwards compatibility with h5py < 3.0 is kept by decoding vlen strings per
+# default. This will change in future versions for consistency with h5py >= 3.0.
+# To silence this warning set kwarg ``decode_vlen_strings=False`` which will
+# return Python bytes from variables containing vlen strings. Setting
+# ``decode_vlen_strings=True`` forces vlen string decoding which returns Python
+# strings from variables containing vlen strings.
+
+
 def print_memory_usage_seq(message, flush=None):
     mem = get_memory_usage()
     print(message, f"{mem/1024: 7.3f} Go", flush=flush)
@@ -342,6 +356,17 @@ def load_state_phys_file(
     return sim
 
 
+def _path_file_from_time_approx(thing, t_approx):
+    if thing is not None and Path(thing).is_file():
+        path_file = Path(thing)
+    else:
+        path_dir = pathdir_from_namedir(thing)
+        # choose the file with the time closer to t_approx
+        name_file = name_file_from_time_approx(path_dir, t_approx)
+        path_file = Path(path_dir) / name_file
+    return path_file
+
+
 def load_for_restart(name_dir=None, t_approx="last", merge_missing_params=False):
     """Load params and Simul for a restart.
 
@@ -368,15 +393,11 @@ def load_for_restart(name_dir=None, t_approx="last", merge_missing_params=False)
 
     """
 
-    path_dir = pathdir_from_namedir(name_dir)
+    path_file = _path_file_from_time_approx(name_dir, t_approx)
+    path_dir = path_file.parent
+
     solver = _import_solver_from_path(path_dir)
-    Simul = solver.Simul
-
-    # choose the file with the time closer to t_approx
-    name_file = name_file_from_time_approx(path_dir, t_approx)
-    path_file = os.path.join(path_dir, name_file)
-
-    Simul = _extend_simul_class_from_path(Simul, path_file)
+    Simul = _extend_simul_class_from_path(solver.Simul, path_file)
 
     if merge_missing_params:
         # this has to be done by all processes otherwise there is a problem
@@ -542,14 +563,13 @@ def modif_resolution_from_dir_memory_efficient(
     if mpi.nb_proc > 1:
         raise NotImplementedError
 
-    path_dir = pathdir_from_namedir(name_dir)
-    solver = _import_solver_from_path(path_dir)
-
-    name_file = name_file_from_time_approx(path_dir, t_approx)
-    path_file = path_dir / name_file
+    path_file = _path_file_from_time_approx(name_dir, t_approx)
     print(f"Changing resolution of the state contained in\n{path_file}")
+    path_dir = path_file.parent
 
+    solver = _import_solver_from_path(path_dir)
     Simul = _extend_simul_class_from_path(solver.Simul, path_file)
+
     try:
         info_solver = Simul.info_solver
     except AttributeError:
@@ -700,3 +720,16 @@ def open_patient(
                 raise
             time.sleep(time_wait_once)
     return file
+
+
+def ensure_radians(angle):
+    """Convert strings like "45°" to radians (as float)"""
+    if isinstance(angle, str):
+        if angle.endswith("°"):
+            angle = radians(float(angle[:-1]))
+        else:
+            raise ValueError(
+                "Angle should be a string with \n"
+                + "the degree symbol or a float in radians"
+            )
+    return angle
